@@ -133,11 +133,7 @@ static void render(int isdb, char *pre, int rflags, const char *pptxt,
 			}
 		}
 		else {
-			/*
-			 * Additional NULL parameter below works around
-			 * fatal error when compiled with -Werror=format-security.
-			 */
-			printf(txt[isdb], NULL);	/* No args */
+			printf("%s", txt[isdb]);
 		}
 	}
 
