@@ -607,14 +607,20 @@ void print_report_hdr(unsigned int flags, struct tm *rectime,
  *
  * RETURNS:
  * Position of current network interface in array of sample statistics used
- * as reference.
+ * as reference, or -1 if it is a new interface (it was not present in array
+ * of stats used as reference).
+ *
+ * Note: A newly registered interface, if it is supernumerary, may make the
+ * last interface in the array going out of the list. Yet an interface going
+ * out of the list still exists in the /proc/net/dev file. Should it go back
+ * in the list (e.g. if some other interfaces have been unregistered) then
+ * its counters will jump as if starting from zero.
  ***************************************************************************
  */
-unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
-			       unsigned int pos)
+int check_net_dev_reg(struct activity *a, int curr, int ref, int pos)
 {
 	struct stats_net_dev *sndc, *sndp;
-	unsigned int index = 0;
+	int index = 0;
 
 	sndc = (struct stats_net_dev *) ((char *) a->buf[curr] + pos * a->msize);
 
@@ -687,23 +693,8 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
 		index++;
 	}
 
-	/* Network interface not found: Look for the first free structure */
-	for (index = 0; index < a->nr; index++) {
-		sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
-		if (!strcmp(sndp->interface, ""))
-			break;
-	}
-	if (index >= a->nr) {
-		/* No free structure: Default is structure of same rank */
-		index = pos;
-	}
-
-	sndp = (struct stats_net_dev *) ((char *) a->buf[ref] + index * a->msize);
-	/* Since the name is not the same, reset all the structure */
-	memset(sndp, 0, STATS_NET_DEV_SIZE);
-	strncpy(sndp->interface, sndc->interface, MAX_IFACE_LEN - 1);
-
-	return  index;
+	/* This is a newly registered interface */
+	return -1;
 }
 
 /*
@@ -719,14 +710,13 @@ unsigned int check_net_dev_reg(struct activity *a, int curr, int ref,
  *
  * RETURNS:
  * Position of current network interface in array of sample statistics used
- * as reference.
+ * as reference, or -1 if it is a newly registered interface.
  ***************************************************************************
  */
-unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
-				unsigned int pos)
+int check_net_edev_reg(struct activity *a, int curr, int ref, int pos)
 {
 	struct stats_net_edev *snedc, *snedp;
-	unsigned int index = 0;
+	int index = 0;
 
 	snedc = (struct stats_net_edev *) ((char *) a->buf[curr] + pos * a->msize);
 
@@ -759,23 +749,8 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
 		index++;
 	}
 
-	/* Network interface not found: Look for the first free structure */
-	for (index = 0; index < a->nr; index++) {
-		snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
-		if (!strcmp(snedp->interface, ""))
-			break;
-	}
-	if (index >= a->nr) {
-		/* No free structure: Default is structure of same rank */
-		index = pos;
-	}
-
-	snedp = (struct stats_net_edev *) ((char *) a->buf[ref] + index * a->msize);
-	/* Since the name is not the same, reset all the structure */
-	memset(snedp, 0, STATS_NET_EDEV_SIZE);
-	strncpy(snedp->interface, snedc->interface, MAX_IFACE_LEN - 1);
-
-	return  index;
+	/* This is a newly registered interface */
+	return -1;
 }
 
 /*
@@ -790,7 +765,8 @@ unsigned int check_net_edev_reg(struct activity *a, int curr, int ref,
  * @pos		Index on current disk.
  *
  * RETURNS:
- * Position of current disk in array of sample statistics used as reference.
+ * Position of current disk in array of sample statistics used as reference
+ * or -1 if it is a newly registered device.
  ***************************************************************************
  */
 int check_disk_reg(struct activity *a, int curr, int ref, int pos)
@@ -824,24 +800,8 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 		index++;
 	}
 
-	/* Disk not found: Look for the first free structure */
-	for (index = 0; index < a->nr; index++) {
-		sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
-		if (!(sdp->major + sdp->minor))
-			break;
-	}
-	if (index >= a->nr) {
-		/* No free structure found: Default is structure of same rank */
-		index = pos;
-	}
-
-	sdp = (struct stats_disk *) ((char *) a->buf[ref] + index * a->msize);
-	/* Since the device is not the same, reset all the structure */
-	memset(sdp, 0, STATS_DISK_SIZE);
-	sdp->major = sdc->major;
-	sdp->minor = sdc->minor;
-
-	return index;
+	/* This is a newly registered device */
+	return -1;
 }
 
 /*
@@ -1566,8 +1526,7 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			 * has AO_MULTIPLE_OUTPUTS flag set.
 			 */
 			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
-			act[p]->opt_flags |= AO_F_MEM_AMT + AO_F_MEM_DIA +
-					     AO_F_MEM_SWAP + AO_F_MEM_ALL;
+			act[p]->opt_flags |= AO_F_MEMORY + AO_F_SWAP + AO_F_MEM_ALL;
 
 			p = get_activity_position(act, A_IRQ, EXIT_IF_NOT_FOUND);
 			memset(act[p]->bitmap->b_array, ~0,
@@ -1651,7 +1610,7 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 		case 'r':
 			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
 			act[p]->options   |= AO_SELECTED;
-			act[p]->opt_flags |= AO_F_MEM_AMT;
+			act[p]->opt_flags |= AO_F_MEMORY;
 			if (!*(argv[*opt] + i + 1) && argv[*opt + 1] && !strcmp(argv[*opt + 1], K_ALL)) {
 				(*opt)++;
 				act[p]->opt_flags |= AO_F_MEM_ALL;
@@ -1659,16 +1618,10 @@ int parse_sar_opt(char *argv[], int *opt, struct activity *act[],
 			}
 			break;
 
-		case 'R':
-			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
-			act[p]->options   |= AO_SELECTED;
-			act[p]->opt_flags |= AO_F_MEM_DIA;
-			break;
-
 		case 'S':
 			p = get_activity_position(act, A_MEMORY, EXIT_IF_NOT_FOUND);
 			act[p]->options   |= AO_SELECTED;
-			act[p]->opt_flags |= AO_F_MEM_SWAP;
+			act[p]->opt_flags |= AO_F_SWAP;
 			break;
 
 		case 't':
